@@ -5,17 +5,22 @@
 #          config values from a given file and
 #          return it as hash structure
 #
-# Copyright (c) 2000-2006 Thomas Linden <tom@daemon.de>.
+# Copyright (c) 2000-2007 Thomas Linden <tlinden |AT| cpan.org>.
 # All Rights Reserved. Std. disclaimer applies.
 # Artificial License, same as perl itself. Have fun.
 #
 # namespace
 package Config::General;
 
+use strict;
+use warnings;
+use English '-no_match_vars';
+
+use IO::File;
 use FileHandle;
 use File::Spec::Functions qw(splitpath file_name_is_absolute catfile catpath);
 use File::Glob qw/:glob/;
-use strict;
+
 
 # on debian with perl > 5.8.4 croak() doesn't work anymore without this.
 # There is some require statement which dies 'cause it can't find Carp::Heavy,
@@ -27,11 +32,11 @@ use Carp::Heavy;
 use Carp;
 use Exporter;
 
-$Config::General::VERSION = "2.31";
+$Config::General::VERSION = 2.33;
 
-use vars  qw(@ISA @EXPORT);
-@ISA    = qw(Exporter);
-@EXPORT = qw(ParseConfig SaveConfig SaveConfigString);
+use vars  qw(@ISA @EXPORT_OK);
+use base qw(Exporter);
+@EXPORT_OK = qw(ParseConfig SaveConfig SaveConfigString);
 
 sub new {
   #
@@ -43,153 +48,53 @@ sub new {
   # define default options
   my $self = {
 	      SlashIsDirectory      => 0,
-
 	      AllowMultiOptions     => 1,
-
 	      MergeDuplicateOptions => 0,
 	      MergeDuplicateBlocks  => 0,
-
 	      LowerCaseNames        => 0,
-
+	      ApacheCompatible      => 0,
 	      UseApacheInclude      => 0,
 	      IncludeRelative       => 0,
 	      IncludeDirectories    => 0,
 	      IncludeGlob           => 0,
-
 	      AutoLaunder           => 0,
-
 	      AutoTrue              => 0,
-
 	      AutoTrueFlags         => {
 					true  => '^(on|yes|true|1)$',
 					false => '^(off|no|false|0)$',
 				       },
-
 	      DefaultConfig         => {},
-
 	      level                 => 1,
-
 	      InterPolateVars       => 0,
-
 	      InterPolateEnv        => 0,
-
 	      ExtendedAccess        => 0,
-
 	      SplitPolicy           => 'guess', # also possible: whitespace, equalsign and custom
-
 	      SplitDelimiter        => 0,       # must be set by the user if SplitPolicy is 'custom'
-
 	      StoreDelimiter        => 0,       # will be set by me unless user uses 'custom' policy
-
 	      CComments             => 1,       # by default turned on
-
 	      BackslashEscape       => 0,       # by default turned off, allows escaping anything using  the backslash
-
 	      StrictObjects         => 1,       # be strict on non-existent keys in OOP mode
-
 	      StrictVars            => 1,       # be strict on undefined variables in Interpolate mode
-
-	      Tie                   => "",      # could be set to a perl module for tie'ing new hashes
-
+	      Tie                   => q(),      # could be set to a perl module for tie'ing new hashes
 	      parsed                => 0,       # internal state stuff for variable interpolation
-	      upperkey              => "",
+	      upperkey              => q(),
 	      upperkeys             => [],
-	      lastkey               => "",
-	      prevkey               => " ",
+	      lastkey               => q(),
+	      prevkey               => q( ),
 	      files                 => {},      # which files we have read, if any
 	     };
 
   # create the class instance
-  bless($self,$class);
-
+  bless $self, $class;
 
   if ($#param >= 1) {
     # use of the new hash interface!
-    my %conf = @param;
-
-    # save the parameter list for ::Extended's new() calls
-    $self->{Params} = \%conf;
-
-    # be backwards compatible
-    $self->{ConfigFile} = delete $conf{-file} if(exists $conf{-file});
-    $self->{ConfigHash} = delete $conf{-hash} if(exists $conf{-hash});
-
-    # store input, file, handle, or array
-    $self->{ConfigFile} = delete $conf{-ConfigFile} if(exists $conf{-ConfigFile});
-    $self->{ConfigHash} = delete $conf{-ConfigHash} if(exists $conf{-ConfigHash});
-
-    # store search path for relative configs, if any
-    if (exists $conf{-ConfigPath}) {
-      my $configpath = delete $conf{-ConfigPath};
-      $self->{ConfigPath} = ref $configpath eq 'ARRAY' ? $configpath : [$configpath];
-    }
-
-    # handle options which contains values we are needing (strings, hashrefs or the like)
-    if (exists $conf{-String} ) {
-      if ($conf{-String}) {
-	$self->{StringContent} = $conf{-String};
-      }
-      delete $conf{-String};
-    }
-
-    if (exists $conf{-Tie}) {
-      if ($conf{-Tie}) {
-	$self->{Tie} = delete $conf{-Tie};
-	$self->{DefaultConfig} = $self->_hashref();
-      }
-    }
-
-    if (exists $conf{-FlagBits}) {
-      if ($conf{-FlagBits} && ref($conf{-FlagBits}) eq "HASH") {
-	$self->{FlagBits} = 1;
-	$self->{FlagBitsFlags} = $conf{-FlagBits};
-      }
-      delete $conf{-FlagBits};
-    }
-
-    if (exists $conf{-DefaultConfig}) {
-      if ($conf{-DefaultConfig} && ref($conf{-DefaultConfig}) eq "HASH") {
-	$self->{DefaultConfig} = $conf{-DefaultConfig};
-      }
-      elsif ($conf{-DefaultConfig} && ref($conf{-DefaultConfig}) eq "") {
-	$self->_read($conf{-DefaultConfig}, "SCALAR");
-	$self->{DefaultConfig} = $self->_parse($self->_hashref(), $self->{content});
-	$self->{content} = ();
-      }
-      delete $conf{-DefaultConfig};
-    }
-
-    # handle options which may either be true or false
-    # allowing "human" logic about what is true and what is not
-    foreach my $entry (keys %conf) {
-      my $key = $entry;
-      $key =~ s/^\-//;
-      if (! exists $self->{$key}) {
-	croak "Unknown parameter: $entry => \"$conf{$entry}\" (key: <$key>)\n";
-      }
-      if ($conf{$entry} =~ /$self->{AutoTrueFlags}->{true}/io) {
-	$self->{$key} = 1;
-      }
-      elsif ($conf{$entry} =~ /$self->{AutoTrueFlags}->{false}/io) {
-	$self->{$key} = 0;
-      }
-      else {
-	# keep it untouched
-	$self->{$key} = $conf{$entry};
-      }
-    }
-
-    if ($self->{MergeDuplicateOptions}) {
-      # override if not set by user
-      if (! exists $conf{-AllowMultiOptions}) {
-	$self->{AllowMultiOptions} = 0;
-      }
-    }
+    $self->_prepare(@param);
   }
   elsif ($#param == 0) {
     # use of the old style
     $self->{ConfigFile} = $param[0];
-    if (ref($self->{ConfigFile}) eq "HASH") {
+    if (ref($self->{ConfigFile}) eq 'HASH') {
       $self->{ConfigHash} = delete $self->{ConfigFile};
     }
   }
@@ -199,15 +104,138 @@ sub new {
     $self->{parsed} = 1;
   }
 
-  # prepare the split delimiter if needed
+  # find split policy to use for option/value separation
+  $self->_splitpolicy();
+
+  # bless into variable interpolation module if neccessary
+  $self->_blessvars();
+
+  # process as usual
+  if (!$self->{parsed}) {
+    $self->_process();
+  }
+
+  # bless into OOP namespace if required
+  $self->_blessoop();
+
+  return $self;
+}
+
+
+
+sub _process {
+  #
+  # call _read() and _parse() on the given config
+  my($self) = @_;
+
+  if ($self->{DefaultConfig} && $self->{InterPolateVars}) {
+    $self->{DefaultConfig} = $self->_interpolate_hash($self->{DefaultConfig}); # FIXME: _hashref() ?
+  }
+  if (exists $self->{StringContent}) {
+    # consider the supplied string as config file
+    $self->_read($self->{StringContent}, 'SCALAR');
+    $self->{config} = $self->_parse($self->{DefaultConfig}, $self->{content});
+  }
+  elsif (exists $self->{ConfigHash}) {
+    if (ref($self->{ConfigHash}) eq 'HASH') {
+      # initialize with given hash
+      $self->{config} = $self->{ConfigHash};
+      $self->{parsed} = 1;
+    }
+    else {
+      croak "Parameter -ConfigHash must be a hash reference!\n";
+    }
+  }
+  elsif (ref($self->{ConfigFile}) eq 'GLOB' || ref($self->{ConfigFile}) eq 'FileHandle') {
+    # use the file the glob points to
+    $self->_read($self->{ConfigFile});
+    $self->{config} = $self->_parse($self->{DefaultConfig}, $self->{content});
+  }
+  else {
+    if ($self->{ConfigFile}) {
+      # open the file and read the contents in
+      $self->{configfile} = $self->{ConfigFile};
+      if ( file_name_is_absolute($self->{ConfigFile}) ) {
+	# look if is is an absolute path and save the basename if it is absolute
+	my ($volume, $path, undef) = splitpath($self->{ConfigFile});
+	$path =~ s#/$##; # remove eventually existing trailing slash
+	if (! $self->{ConfigPath}) {
+	  $self->{ConfigPath} = [];
+	}
+	unshift @{$self->{ConfigPath}}, catpath($volume, $path, q());
+      }
+      $self->_open($self->{configfile});
+      # now, we parse immdediately, getall simply returns the whole hash
+      $self->{config} = $self->_hashref();
+      $self->{config} = $self->_parse($self->{DefaultConfig}, $self->{content});
+    }
+    else {
+      # hm, no valid config file given, so try it as an empty object
+      $self->{config} = $self->_hashref();
+      $self->{parsed} = 1;
+    }
+  }
+}
+
+
+sub _blessoop {
+  #
+  # bless into ::Extended if neccessary
+  my($self) = @_;
+  if ($self->{ExtendedAccess}) {
+    # we are blessing here again, to get into the ::Extended namespace
+    # for inheriting the methods available overthere, which we doesn't have.
+    bless $self, 'Config::General::Extended';
+    eval {
+      require Config::General::Extended;
+    };
+    if ($EVAL_ERROR) {
+      croak $EVAL_ERROR;
+    }
+  }
+#  return $self;
+}
+
+sub _blessvars {
+  #
+  # bless into ::Interpolated if neccessary
+  my($self) = @_;
+  if ($self->{InterPolateVars} || $self->{InterPolateEnv}) {
+    # InterPolateEnv implies InterPolateVars
+    $self->{InterPolateVars} = 1;
+
+    # we are blessing here again, to get into the ::InterPolated namespace
+    # for inheriting the methods available overthere, which we doesn't have here.
+    bless $self, 'Config::General::Interpolated';
+    eval {
+      require Config::General::Interpolated;
+    };
+    if ($EVAL_ERROR) {
+      croak $EVAL_ERROR;
+    }
+    # pre-compile the variable regexp
+    $self->{regex} = $self->_set_regex();
+  }
+#  return $self;
+}
+
+
+sub _splitpolicy {
+  #
+  # find out what split policy to use
+  my($self) = @_;
   if ($self->{SplitPolicy} ne 'guess') {
     if ($self->{SplitPolicy} eq 'whitespace') {
       $self->{SplitDelimiter} = '\s+';
-      $self->{StoreDelimiter} = "   " if(!$self->{StoreDelimiter});
+      if (!$self->{StoreDelimiter}) {
+	$self->{StoreDelimiter} = q(   );
+      }
     }
     elsif ($self->{SplitPolicy} eq 'equalsign') {
       $self->{SplitDelimiter} = '\s*=\s*';
-      $self->{StoreDelimiter} = " = " if(!$self->{StoreDelimiter});
+      if (!$self->{StoreDelimiter}) {
+	$self->{StoreDelimiter} = ' = ';
+      }
     }
     elsif ($self->{SplitPolicy} eq 'custom') {
       if (! $self->{SplitDelimiter} ) {
@@ -219,97 +247,123 @@ sub new {
     }
   }
   else {
-    $self->{StoreDelimiter} = "   " if(!$self->{StoreDelimiter});
-  }
-
-  if ($self->{InterPolateVars} || $self->{InterPolateEnv}) {
-    # InterPolateEnv implies InterPolateVars
-    $self->{InterPolateVars} = 1;
-
-    # we are blessing here again, to get into the ::InterPolated namespace
-    # for inheriting the methods available overthere, which we doesn't have.
-    #
-    bless($self, "Config::General::Interpolated");
-    eval {
-      require Config::General::Interpolated;
-    };
-    if ($@) {
-      croak $@;
-    }
-    # pre-compile the variable regexp
-    $self->{regex} = $self->_set_regex();
-  }
-
-  # process as usual
-  if (!$self->{parsed}) {
-    if ($self->{DefaultConfig} && $self->{InterPolateVars}) {
-      $self->{DefaultConfig} = $self->_interpolate_hash($self->{DefaultConfig}); # FIXME: _hashref() ?
-    }
-    if (exists $self->{StringContent}) {
-      # consider the supplied string as config file
-      $self->_read($self->{StringContent}, "SCALAR");
-      $self->{config} = $self->_parse($self->{DefaultConfig}, $self->{content});
-    }
-    elsif (exists $self->{ConfigHash}) {
-      if (ref($self->{ConfigHash}) eq "HASH") {
-	# initialize with given hash
-	$self->{config} = $self->{ConfigHash};
-	$self->{parsed} = 1;
-      }
-      else {
-	croak "Parameter -ConfigHash must be a hash reference!\n";
-      }
-    }
-    elsif (ref($self->{ConfigFile}) eq "GLOB" || ref($self->{ConfigFile}) eq "FileHandle") {
-      # use the file the glob points to
-      $self->_read($self->{ConfigFile});
-      $self->{config} = $self->_parse($self->{DefaultConfig}, $self->{content});
-    }
-    else {
-      if ($self->{ConfigFile}) {
-	# open the file and read the contents in
-	$self->{configfile} = $self->{ConfigFile};
-	if ( file_name_is_absolute($self->{ConfigFile}) ) {
-	  # look if is is an absolute path and save the basename if it is absolute
-	  my ($volume, $path, undef) = splitpath($self->{ConfigFile});
-	  $path =~ s#/$##; # remove eventually existing trailing slash
-	  $self->{ConfigPath} = [] unless $self->{ConfigPath};
-	  unshift @{$self->{ConfigPath}}, catpath($volume, $path, '');
-	}
-	$self->_open($self->{configfile});
-	# now, we parse immdediately, getall simply returns the whole hash
-	$self->{config} = $self->_hashref();
-	$self->{config} = $self->_parse($self->{DefaultConfig}, $self->{content});
-      }
-      else {
-	# hm, no valid config file given, so try it as an empty object
-	$self->{config} = $self->_hashref();
-	$self->{parsed} = 1;
-      }
+    if (!$self->{StoreDelimiter}) {
+      $self->{StoreDelimiter} = q(   );
     }
   }
-
-  #
-  # Submodule handling. Parsing is already done at this point.
-  #
-  if ($self->{ExtendedAccess}) {
-    #
-    # we are blessing here again, to get into the ::Extended namespace
-    # for inheriting the methods available overthere, which we doesn't have.
-    #
-    bless($self, "Config::General::Extended");
-    eval {
-      require Config::General::Extended;
-    };
-    if ($@) {
-      croak $@;
-    }
-  }
-
-  return $self;
 }
 
+sub _prepare {
+  #
+  # prepare the class parameters, mangle them, if there
+  # are options to reset or to override, do it here.
+  my ($self, %conf) = @_;
 
+  # save the parameter list for ::Extended's new() calls
+  $self->{Params} = \%conf;
+
+  # be backwards compatible
+  if (exists $conf{-file}) {
+    $self->{ConfigFile} = delete $conf{-file};
+  }
+  if (exists $conf{-hash}) {
+    $self->{ConfigHash} = delete $conf{-hash};
+  }
+
+  # store input, file, handle, or array
+  if (exists $conf{-ConfigFile}) {
+    $self->{ConfigFile} = delete $conf{-ConfigFile};
+  }
+  if (exists $conf{-ConfigHash}) {
+    $self->{ConfigHash} = delete $conf{-ConfigHash};
+  }
+
+  # store search path for relative configs, if any
+  if (exists $conf{-ConfigPath}) {
+    my $configpath = delete $conf{-ConfigPath};
+    $self->{ConfigPath} = ref $configpath eq 'ARRAY' ? $configpath : [$configpath];
+  }
+
+  # handle options which contains values we need (strings, hashrefs or the like)
+  if (exists $conf{-String} ) {
+    if (ref(\$conf{-String}) eq 'SCALAR') {
+      if ( $conf{-String}) {
+	$self->{StringContent} = $conf{-String};
+      }
+      delete $conf{-String};
+    }
+    else {
+      croak "Parameter -String must be a SCALAR!\n";
+    }
+  }
+
+  if (exists $conf{-Tie}) {
+    if ($conf{-Tie}) {
+      $self->{Tie} = delete $conf{-Tie};
+      $self->{DefaultConfig} = $self->_hashref();
+    }
+  }
+
+  if (exists $conf{-FlagBits}) {
+    if ($conf{-FlagBits} && ref($conf{-FlagBits}) eq 'HASH') {
+      $self->{FlagBits} = 1;
+      $self->{FlagBitsFlags} = $conf{-FlagBits};
+    }
+    delete $conf{-FlagBits};
+  }
+
+  if (exists $conf{-DefaultConfig}) {
+    if ($conf{-DefaultConfig} && ref($conf{-DefaultConfig}) eq 'HASH') {
+      $self->{DefaultConfig} = $conf{-DefaultConfig};
+    }
+    elsif ($conf{-DefaultConfig} && ref($conf{-DefaultConfig}) eq q()) {
+      $self->_read($conf{-DefaultConfig}, 'SCALAR');
+      $self->{DefaultConfig} = $self->_parse($self->_hashref(), $self->{content});
+      $self->{content} = ();
+    }
+    delete $conf{-DefaultConfig};
+  }
+
+  # handle options which may either be true or false
+  # allowing "human" logic about what is true and what is not
+  foreach my $entry (keys %conf) {
+    my $key = $entry;
+    $key =~ s/^\-//;
+    if (! exists $self->{$key}) {
+      croak "Unknown parameter: $entry => \"$conf{$entry}\" (key: <$key>)\n";
+    }
+    if ($conf{$entry} =~ /$self->{AutoTrueFlags}->{true}/io) {
+      $self->{$key} = 1;
+    }
+    elsif ($conf{$entry} =~ /$self->{AutoTrueFlags}->{false}/io) {
+      $self->{$key} = 0;
+    }
+    else {
+      # keep it untouched
+      $self->{$key} = $conf{$entry};
+    }
+  }
+
+  if ($self->{MergeDuplicateOptions}) {
+    # override if not set by user
+    if (! exists $conf{-AllowMultiOptions}) {
+      $self->{AllowMultiOptions} = 0;
+    }
+  }
+
+  if ($self->{ApacheCompatible}) {
+    # turn on all apache compatibility options which has
+    # been incorporated during the years...
+    $self->{UseApacheInclude}   = 1;
+    $self->{IncludeRelative}    = 1;
+    $self->{IncludeDirectories} = 1;
+    $self->{IncludeGlob}        = 1;
+    $self->{SpashIsDirectory}   = 1;
+    $self->{SplitPolicy}        = 'whitespace';
+    $self->{CComments}          = 0;
+    $self->{BackslashEscape}    = 1;
+  }
+}
 
 sub getall {
   #
@@ -334,16 +388,18 @@ sub _open {
   # open the config file, or expand a directory or glob
   #
   my($this, $configfile) = @_;
-  my $fh = new FileHandle;
+  my $fh;
 
   if ($this->{IncludeGlob} and $configfile =~ /[*?\[\{\\]/) {
     # Something like: *.conf (or maybe dir/*.conf) was included; expand it and
     # pass each expansion through this method again.
-    my @include = grep -f, bsd_glob($configfile, GLOB_BRACE | GLOB_QUOTE);
+    my @include = grep { -f $_ } bsd_glob($configfile, GLOB_BRACE | GLOB_QUOTE);
     if (@include == 1) {
       $configfile = $include[0];
     }
-    else { # Multiple results or no expansion results (which is fine, include foo/* shouldn't fail if there isn't anything matching)
+    else {
+      # Multiple results or no expansion results (which is fine,
+      # include foo/* shouldn't fail if there isn't anything matching)
       local $this->{IncludeGlob};
       for (@include) {
 	$this->_open($_);
@@ -354,7 +410,7 @@ sub _open {
 
   if (!-e $configfile) {
     my $found;
-    if (defined($this->{ConfigPath})) {
+    if (defined $this->{ConfigPath}) {
       # try to find the file within ConfigPath
       foreach my $dir (@{$this->{ConfigPath}}) {
 	if( -e catfile($dir, $configfile) ) {
@@ -365,27 +421,27 @@ sub _open {
       }
     }
     if (!$found) {
-      my $path_message = defined $this->{ConfigPath} ? ' within ConfigPath: ' . join('.', @{$this->{ConfigPath}}) : '';
+      my $path_message = defined $this->{ConfigPath} ? q( within ConfigPath: ) . join(q(.), @{$this->{ConfigPath}}) : q();
       croak qq{The file "$configfile" does not exist$path_message!};
     }
   }
 
-  local ($/) = $/;
-  unless ($/) {
-    carp("\$/ (INPUT_RECORD_SEPARATOR) is undefined.  Guessing you want a line feed character");
-    $/ = "\n";
+  local ($RS) = $RS;
+  if (! $RS) {
+    carp(q(\$RS (INPUT_RECORD_SEPARATOR) is undefined.  Guessing you want a line feed character));
+    $RS = "\n";
   }
 
   if (-d $configfile and $this->{IncludeDirectories}) {
     # A directory was included; include all the files inside that directory in ASCII order
     local *INCLUDEDIR;
     opendir INCLUDEDIR, $configfile or croak "Could not open directory $configfile!($!)\n";
-    my @files = sort grep -f "$configfile/$_", readdir INCLUDEDIR;
+    my @files = sort grep { -f "$configfile/$_" } "$configfile/$_", readdir INCLUDEDIR;
     closedir INCLUDEDIR;
     local $this->{CurrentConfigFilePath} = $configfile;
     for (@files) {
-      unless ($this->{files}->{"$configfile/$_"}) {
-	open $fh, "<$configfile/$_" or croak "Could not open $configfile/$_!($!)\n";
+      if (! $this->{files}->{"$configfile/$_"}) {
+	$fh = IO::File->new( "$configfile/$_", 'r') or croak "Could not open $configfile/$_!($!)\n";
 	$this->{files}->{"$configfile/$_"} = 1;
 	$this->_read($fh);
       }
@@ -398,16 +454,17 @@ sub _open {
       return;
     }
     else {
-      open $fh, "<$configfile" or croak "Could not open $configfile!($!)\n";
+      $fh = IO::File->new( "$configfile", 'r') or croak "Could not open $configfile!($!)\n";
 
       $this->{files}->{$configfile}    = 1;
 
       my ($volume, $path, undef)           = splitpath($configfile);
-      local $this->{CurrentConfigFilePath} = catpath($volume, $path, '');
+      local $this->{CurrentConfigFilePath} = catpath($volume, $path, q());
 
       $this->_read($fh);
     }
   }
+  return;
 }
 
 
@@ -419,14 +476,14 @@ sub _read {
   #
   my($this, $fh, $flag) = @_;
   my(@stuff, @content, $c_comment, $longline, $hier, $hierend, @hierdoc);
-  local $_;
+  local $_ = q();
 
-  if ($flag && $flag eq "SCALAR") {
-    if (ref($fh) eq "ARRAY") {
+  if ($flag && $flag eq 'SCALAR') {
+    if (ref($fh) eq 'ARRAY') {
       @stuff = @{$fh};
     }
     else {
-      @stuff = split "\n", $fh;
+      @stuff = split /\n/, $fh;
     }
   }
   else {
@@ -435,8 +492,9 @@ sub _read {
 
   foreach (@stuff) {
     if ($this->{AutoLaunder}) {
-      m/^(.*)$/;
-      $_ = $1;
+      if (m/^(.*)$/) {
+	$_ = $1;
+      }
     }
 
     chomp;
@@ -472,7 +530,7 @@ sub _read {
       # inside here-doc, only look for $hierend marker
       if (/^(\s*)\Q$hierend\E\s*$/) {
 	my $indent = $1;                 # preserve indentation
-	$hier .= " " . chr(182);         # append a "" to the here-doc-name, so
+	$hier .= ' ' . chr 182;         # append a "" to the here-doc-name, so
 	                                 # _parse will also preserver indentation
 	if ($indent) {
 	  foreach (@hierdoc) {
@@ -506,7 +564,7 @@ sub _read {
 
 
     # look for multiline option, indicated by a trailing backslash
-    my $extra = $this->{BackslashEscape} ? '(?<!\\\\)' : '';
+    my $extra = $this->{BackslashEscape} ? '(?<!\\\\)' : q();
     if (/$extra\\$/) {
       chop;
       s/^\s*//;
@@ -525,7 +583,7 @@ sub _read {
 
 
     # transform explicit-empty blocks to conforming blocks
-    if (/\s*<([^\/]+?.*?)\/>$/) {
+    if (!$this->{ApacheCompatible} && /\s*<([^\/]+?.*?)\/>$/) {
       my $block = $1;
       if ($block !~ /\"/) {
 	if ($block !~ /\s[^\s]/) {
@@ -581,11 +639,11 @@ sub _read {
     else {
       # look for include statement(s)
       my $incl_file;
-      my $path = "";
-      if ( $this->{IncludeRelative} and defined($this->{CurrentConfigFilePath})) {
+      my $path = '';
+      if ( $this->{IncludeRelative} and defined $this->{CurrentConfigFilePath}) {
       	$path = $this->{CurrentConfigFilePath};
       }
-      elsif (defined($this->{ConfigPath})) {
+      elsif (defined $this->{ConfigPath}) {
 	# fetch pathname of base config file, assuming the 1st one is the path of it
 	$path = $this->{ConfigPath}->[0];
       }
@@ -621,7 +679,7 @@ sub _parse {
   my($this, $config, $content) = @_;
   my(@newcontent, $block, $blockname, $chunk,$block_level);
   local $_;
-  my $indichar = chr(182);  # , inserted by _open, our here-doc indicator
+  my $indichar = chr 182;  # , inserted by _open, our here-doc indicator
 
   foreach (@{$content}) {                                  # loop over content stack
     chomp;
@@ -671,11 +729,13 @@ sub _parse {
 	  # interpolate block(name), add "<" and ">" to the key, because
 	  # it is sure that such keys does not exist otherwise.
 	  $block     = $this->_interpolate("<$block>", $block);
-	  if ($blockname) {
-	    $blockname = $this->_interpolate("<$blockname>", $blockname);
+	  if (defined $blockname) {
+	    $blockname = $this->_interpolate("<$blockname>", "$blockname");
 	  }
 	}
-	$block = lc($block) if $this->{LowerCaseNames};    # only for blocks lc(), if configured via new()
+	if ($this->{LowerCaseNames}) {
+	  $block = lc $block;    # only for blocks lc(), if configured via new()
+	}
 	$this->{level} += 1;
 	undef @newcontent;
 	next;
@@ -684,7 +744,9 @@ sub _parse {
 	croak "EndBlock \"<\/$1>\" has no StartBlock statement (level: $this->{level}, chunk $chunk)!\n";
       }
       else {                                               # insert key/value pair into actual node
-	$option = lc($option) if $this->{LowerCaseNames};
+	if ($this->{LowerCaseNames}) {
+	  $option = lc $option;
+	}
 	if (exists $config->{$option}) {
 	  if ($this->{MergeDuplicateOptions}) {
 	    $config->{$option} = $this->_parse_value($option, $value);
@@ -696,7 +758,7 @@ sub _parse {
 	    }
 	    else {
 	      # yes, duplicates allowed
-	      if (ref($config->{$option}) ne "ARRAY") {      # convert scalar to array
+	      if (ref($config->{$option}) ne 'ARRAY') {      # convert scalar to array
 		my $savevalue = $config->{$option};
 		delete $config->{$option};
 		push @{$config->{$option}}, $savevalue;
@@ -705,7 +767,7 @@ sub _parse {
 		# check if arrays are supported by the underlying hash
 		my $i = scalar @{$config->{$option}};
 	      };
-	      if ($@) {
+	      if ($EVAL_ERROR) {
 		$config->{$option} = $this->_parse_value($option, $value);
 	      }
 	      else {
@@ -731,11 +793,12 @@ sub _parse {
 	push @newcontent, $_;       # push onto new content stack
       }
       else {                        # calling myself recursively, end of $block reached, $block_level is 0
-	if ($blockname) {           # a named block, make it a hashref inside a hash within the current node
+	if (defined $blockname) {           # a named block, make it a hashref inside a hash within the current node
 	  $this->_savelast($blockname);
 
-	  $config->{$block} = $this->_hashref() # Make sure that the hash is not created implicitely
-	    unless exists $config->{$block};
+	  if (! exists $config->{$block}) {
+	    $config->{$block} = $this->_hashref(); # Make sure that the hash is not created implicitely
+	  }
 
 	  if (exists $config->{$block}->{$blockname}) {    # the named block already exists, make it an array
 	    if ($this->{MergeDuplicateBlocks}) {
@@ -751,7 +814,7 @@ sub _parse {
 		my $savevalue = $config->{$block}->{$blockname};
 		delete $config->{$block}->{$blockname};
 		my @ar;
-		if (ref $savevalue eq "ARRAY") {
+		if (ref $savevalue eq 'ARRAY') {
 		  push @ar, @{$savevalue};                   # preserve array if any
 		}
 		else {
@@ -762,7 +825,7 @@ sub _parse {
 	      }
 	    }
 	  }
-	  elsif (ref($config->{$block}) eq "ARRAY") {
+	  elsif (ref($config->{$block}) eq 'ARRAY') {
 	    croak "Cannot add named block <$block $blockname> to hash! Block <$block> occurs more than once.\n"
 	         ."Turn on -MergeDuplicateBlocks or make sure <$block> occurs only once in the config.\n";
 	  }
@@ -827,15 +890,17 @@ sub _parse {
 
 sub _savelast {
   my($this, $key) = @_;
-  push(@{$this->{upperkeys}}, $this->{lastkey});
+  push @{$this->{upperkeys}}, $this->{lastkey};
   $this->{lastkey}  = $this->{prevkey};
   $this->{prevkey}  = $key;
+  return;
 }
 
 sub _backlast {
   my($this, $key) = @_;
   $this->{prevkey} = $this->{lastkey};
-  $this->{lastkey} = pop(@{$this->{upperkeys}});
+  $this->{lastkey} = pop @{$this->{upperkeys}};
+  return;
 }
 
 sub _parse_value {
@@ -847,7 +912,9 @@ sub _parse_value {
   my($this, $option, $value) =@_;
 
   # avoid "Use of uninitialized value"
-  $value = '' unless defined $value;
+  if (! defined $value) {
+    $value = q();
+  }
 
   if ($this->{InterPolateVars}) {
     $value = $this->_interpolate($option, $value);
@@ -892,7 +959,7 @@ sub NoMultiOptions {
   # Since we do parsing from within new(), we must
   # call it again if one turns NoMultiOptions on!
   #
-  croak "The NoMultiOptions() method is deprecated. Set 'AllowMultiOptions' to 'no' instead!";
+  croak q(The NoMultiOptions() method is deprecated. Set 'AllowMultiOptions' to 'no' instead!);
 }
 
 
@@ -912,8 +979,9 @@ sub save {
     $this->save_file($one, \%h);
   }
   else {
-    croak "The save() method is deprecated. Use the new save_file() method instead!";
+    croak q(The save() method is deprecated. Use the new save_file() method instead!);
   }
+  return;
 }
 
 
@@ -922,14 +990,14 @@ sub save_file {
   # save the config back to disk
   #
   my($this, $file, $config) = @_;
-  my $fh = new FileHandle;
+  my $fh;
   my $config_string;
 
   if (!$file) {
     croak "Filename is required!";
   }
   else {
-    open $fh, ">$file" or croak "Could not open $file!($!)\n";
+    $fh = IO::File->new( "$file", 'w') or croak "Could not open $file!($!)\n";
 
     if (!$config) {
       if (exists $this->{config}) {
@@ -944,15 +1012,16 @@ sub save_file {
     }
 
     if ($config_string) {
-      print $fh $config_string;
+      print {$fh} $config_string;
     }
     else {
       # empty config for whatever reason, I don't care
-      print $fh "";
+      print {$fh} q();
     }
 
     close $fh;
   }
+  return;
 }
 
 
@@ -963,7 +1032,7 @@ sub save_string {
   #
   my($this, $config) = @_;
 
-  if (!$config || ref($config) ne "HASH") {
+  if (!$config || ref($config) ne 'HASH') {
     if (exists $this->{config}) {
       return $this->_store(0, %{$this->{config}});
     }
@@ -974,6 +1043,7 @@ sub save_string {
   else {
     return $this->_store(0, %{$config});
   }
+  return;
 }
 
 
@@ -984,14 +1054,14 @@ sub _store {
   #
   my($this, $level, %config) = @_;
   local $_;
-  my $indent = "    " x $level;
+  my $indent = q(    ) x $level;
 
-  my $config_string = "";
+  my $config_string = q();
 
-  foreach my $entry (sort keys %config) {
-    if (ref($config{$entry}) eq "ARRAY") {
+  foreach my $entry (keys %config) {
+    if (ref($config{$entry}) eq 'ARRAY') {
       foreach my $line (@{$config{$entry}}) {
-        if (ref($line) eq "HASH") {
+        if (ref($line) eq 'HASH') {
 	  $config_string .= $this->_write_hash($level, $entry, $line);
         }
         else {
@@ -999,7 +1069,7 @@ sub _store {
         }
       }
     }
-    elsif (ref($config{$entry}) eq "HASH") {
+    elsif (ref($config{$entry}) eq 'HASH') {
       $config_string .= $this->_write_hash($level, $entry, $config{$entry});
     }
     else {
@@ -1018,18 +1088,18 @@ sub _write_scalar {
   #
   my($this, $level, $entry, $line) = @_;
 
-  my $indent = "    " x $level;
+  my $indent = q(    ) x $level;
 
   my $config_string;
 
   if ($line =~ /\n/ || $line =~ /\\$/) {
     # it is a here doc
     my $delimiter;
-    my $tmplimiter = "EOF";
+    my $tmplimiter = 'EOF';
     while (!$delimiter) {
       # create a unique here-doc identifier
       if ($line =~ /$tmplimiter/s) {
-	$tmplimiter .= "%";
+	$tmplimiter .= q(%);
       }
       else {
 	$delimiter = $tmplimiter;
@@ -1058,17 +1128,17 @@ sub _write_hash {
   #
   my($this, $level, $entry, $line) = @_;
 
-  my $indent = "    " x $level;
+  my $indent = q(    ) x $level;
   my $config_string;
 
   if ($entry =~ /\s/) {
     # quote the entry if it contains whitespaces
-    $entry = '"' . $entry . '"';
+    $entry = q(") . $entry . q(");
   }
 
-  $config_string .= $indent . "<" . $entry . ">\n";
+  $config_string .= $indent . q(<) . $entry . ">\n";
   $config_string .= $this->_store($level + 1, %{$line});
-  $config_string .= $indent . "</" . $entry . ">\n";
+  $config_string .= $indent . q(</) . $entry . ">\n";
 
   return $config_string
 }
@@ -1080,13 +1150,13 @@ sub _hashref {
   #
   my($this) = @_;
   my ($package, $filename, $line, $subroutine, $hasargs,
-      $wantarray, $evaltext, $is_require, $hints, $bitmask) = caller(0);
+      $wantarray, $evaltext, $is_require, $hints, $bitmask) = caller 0;
   if ($this->{Tie}) {
     eval {
-      eval "require $this->{Tie}";
+      eval qq{require $this->{Tie}};
     };
-    if ($@) {
-      croak "Could not create a tied hash of type: " . $this->{Tie} . ": " . $@;
+    if ($EVAL_ERROR) {
+      croak q(Could not create a tied hash of type: ) . $this->{Tie} . q(: ) . $EVAL_ERROR;
     }
     my %hash;
     tie %hash, $this->{Tie};
@@ -1116,16 +1186,17 @@ sub SaveConfig {
   my ($file, $hash) = @_;
 
   if (!$file || !$hash) {
-    croak "SaveConfig(): filename and hash argument required.";
+    croak q{SaveConfig(): filename and hash argument required.};
   }
   else {
-    if (ref($hash) ne "HASH") {
-      croak "The second parameter must be a reference to a hash!";
+    if (ref($hash) ne 'HASH') {
+      croak q(The second parameter must be a reference to a hash!);
     }
     else {
       (new Config::General(-ConfigHash => $hash))->save_file($file);
     }
   }
+  return;
 }
 
 sub SaveConfigString {
@@ -1136,22 +1207,24 @@ sub SaveConfigString {
   my ($hash) = @_;
 
   if (!$hash) {
-    croak "SaveConfigString(): Hash argument required.";
+    croak q{SaveConfigString(): Hash argument required.};
   }
   else {
-    if (ref($hash) ne "HASH") {
-      croak "The parameter must be a reference to a hash!";
+    if (ref($hash) ne 'HASH') {
+      croak q(The parameter must be a reference to a hash!);
     }
     else {
       return (new Config::General(-ConfigHash => $hash))->save_string();
     }
   }
+  return;
 }
 
 
 
 # keep this one
 1;
+__END__
 
 
 
@@ -1188,7 +1261,7 @@ In addition to the capabilities of an apache config file it supports some enhanc
 C-style comments or multiline options.
 
 
-=head1 METHODS
+=head1 SUBROUTINES/METHODS
 
 =over
 
@@ -1607,6 +1680,35 @@ Please note that this is a new option (incorporated in version 2.30),
 it may lead to various unexpected sideeffects or other failures.
 You've been warned.
 
+=item B<-ApacheCompatible>
+
+Over the past years a lot of options has been incorporated
+into Config::General to be able to parse real apache configs.
+
+The new B<-ApacheCompatible> option now makes it possible to
+tweak all options in a way that apache configs can be parsed.
+
+This is called "apache compatibility mode" - if you will ever
+have problems with parsing apache configs without this option
+being set, you'll get no help by me. Thanks :)
+
+The following options will be set:
+
+ UseApacheInclude   = 1
+ IncludeRelative    = 1
+ IncludeDirectories = 1
+ IncludeGlob        = 1
+ SpashIsDirectory   = 1
+ SplitPolicy        = 'equalsign'
+ CComments          = 0
+ BackslashEscape    = 1
+
+Take a look into the particular documentation sections what
+those options are doing.
+
+Beside setting some options it also turns off support for
+explicit empty blocks.
+
 =back
 
 
@@ -2117,6 +2219,10 @@ which is supplied with the Config::General distribution.
 Config::General exports some functions too, which makes it somewhat
 easier to use it, if you like this.
 
+How to import the functions:
+
+ use Config::General qw(ParseConfig SaveConfig SaveConfigString);
+
 =over
 
 =item B<ParseConfig()>
@@ -2158,6 +2264,9 @@ Example:
 
 =back
 
+=head1 CONFIGURATION AND ENVIRONMENT
+
+No environment variables will be used.
 
 =head1 SEE ALSO
 
@@ -2171,26 +2280,38 @@ I recommend you to read the following documentations, which are supplied with pe
  Config::General::Extended      Object oriented interface to parsed configs
  Config::General::Interpolated  Allows to use variables inside config files
 
-=head1 COPYRIGHT
+=head1 LICENSE AND COPYRIGHT
 
-Copyright (c) 2000-2006 Thomas Linden
+Copyright (c) 2000-2007 Thomas Linden
 
 This library is free software; you can redistribute it and/or
 modify it under the same terms as Perl itself.
 
+=head1 BUGS AND LIMITATIONS
 
-=head1 BUGS
+See rt.cpan.org for current bugs, if any.
 
-none known yet.
+=head1 INCOMPATIBILITIES
+
+None known.
+
+=head1 DIAGNOSTICS
+
+To debug Config::General use the perl debugger, see L<perldebug>.
+
+=head1 DEPENDENCIES
+
+Config::General depends on the modules L<FileHandle>,
+L<File::Spec::Functions>, L<File::Glob>, which all are
+shipped with perl.
 
 =head1 AUTHOR
 
-Thomas Linden <tom@daemon.de>
-
+Thomas Linden <tlinden |AT| cpan.org>
 
 =head1 VERSION
 
-2.31
+2.33
 
 =cut
 
